@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 
-from gi.repository import GObject, Peas, Gtk, Gdk # pylint: disable-msg=E0611
+from gi.repository import GLib, GObject # pylint: disable-msg=E0611
+from gi.repository import Peas, Gtk, Gdk # pylint: disable-msg=E0611
 from gi.repository import Gio, Pango, Totem # pylint: disable-msg=E0611
 
-import xmlrpclib
+import xmlrpc.client
 import threading
-import xdg.BaseDirectory
+import zlib
 from os import sep, path, mkdir
 import gettext
 
@@ -16,8 +17,6 @@ gettext.textdomain ("totem")
 D_ = gettext.dgettext
 _ = gettext.gettext
 
-GObject.threads_init ()
-
 USER_AGENT = 'Totem'
 OK200 = '200 OK'
 TOTEM_REMOTE_COMMAND_REPLACE = 14
@@ -251,7 +250,7 @@ class OpenSubtitlesModel (object):
 
         try:
             import locale
-            (language_code, _encoding) = locale.getlocale ()
+            (language_code, _) = locale.getlocale ()
             self.lang = LANGUAGES[language_code.split ('_')[0]]
         except (ImportError, IndexError, AttributeError):
             self.lang = 'eng'
@@ -271,7 +270,7 @@ class OpenSubtitlesModel (object):
             # We have already logged-in before, check the connection
             try:
                 result = self._server.NoOperation (self._token)
-            except (xmlrpclib.Fault, xmlrpclib.ProtocolError):
+            except (xmlrpc.client.Fault, xmlrpc.client.ProtocolError):
                 pass
             if result and result['status'] != OK200:
                 return (True, '')
@@ -279,7 +278,7 @@ class OpenSubtitlesModel (object):
         try:
             result = self._server.LogIn (username, password, self.lang,
                                          USER_AGENT)
-        except (xmlrpclib.Fault, xmlrpclib.ProtocolError):
+        except (xmlrpc.client.Fault, xmlrpc.client.ProtocolError):
             pass
 
         if result and result.get ('status') == OK200:
@@ -319,7 +318,7 @@ class OpenSubtitlesModel (object):
             try:
                 result = self._server.SearchSubtitles (self._token,
                                                        [searchdata])
-            except xmlrpclib.ProtocolError:
+            except xmlrpc.client.ProtocolError:
                 message = _(u'Could not contact the OpenSubtitles website.')
 
             if result.get ('data'):
@@ -346,7 +345,7 @@ class OpenSubtitlesModel (object):
             try:
                 result = self._server.DownloadSubtitles (self._token,
                                                          [subtitle_id])
-            except xmlrpclib.ProtocolError:
+            except xmlrpc.client.ProtocolError:
                 message = error_message
 
             if result and result.get ('status') == OK200:
@@ -356,14 +355,11 @@ class OpenSubtitlesModel (object):
                     self._lock.release ()
                     return (None, error_message)
 
-                import StringIO, gzip, base64
-                subtitle_decoded = base64.decodestring (subtitle64)
-                subtitle_gzipped = StringIO.StringIO (subtitle_decoded)
-                subtitle_gzipped_file = gzip.GzipFile (fileobj=subtitle_gzipped)
+                subtitle_unzipped = zlib.decompress(GLib.base64_decode (subtitle64), 47)
 
                 self._lock.release ()
 
-                return (subtitle_gzipped_file.read (), message)
+                return (subtitle_unzipped, message)
         else:
             message = log_in_message
 
@@ -385,9 +381,6 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         schema = 'org.gnome.totem.plugins.opensubtitles'
         self._settings = Gio.Settings.new (schema)
 
-        self._manager = None
-        self._menu_id = None
-        self._action_group = None
         self._action = None
 
         self._find_button = None
@@ -415,14 +408,13 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         # are related to.
         self._filename = None
 
-        self._manager = self._totem.get_ui_manager ()
         self._append_menu ()
 
         self._totem.connect ('file-opened', self.__on_totem__file_opened)
         self._totem.connect ('file-closed', self.__on_totem__file_closed)
 
         # Obtain the ServerProxy and init the model
-        server = xmlrpclib.Server ('http://api.opensubtitles.org/xml-rpc')
+        server = xmlrpc.client.Server ('http://api.opensubtitles.org/xml-rpc')
         self._model = OpenSubtitlesModel (server)
 
     def do_deactivate (self):
@@ -511,7 +503,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         self._tree_view.connect ('row-activated',
                                self.__on_treeview__row_activate)
 
-    def _show_dialog (self, _action):
+    def _show_dialog (self, params, _):
         if not self._dialog:
             self._build_dialog ()
 
@@ -520,34 +512,14 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         self._progress.set_fraction (0.0)
 
     def _append_menu (self):
-        self._action_group = Gtk.ActionGroup (name='OpenSubtitles')
-
-        tooltip_text = _(u"Download movie subtitles from OpenSubtitles")
-        self._action = Gtk.Action (name='opensubtitles',
-                                 label=_(u'_Download Movie Subtitles…'),
-                                 tooltip=tooltip_text,
-                                 stock_id=None)
-
-        self._action_group.add_action (self._action)
-
-        self._manager.insert_action_group (self._action_group, 0)
-
-        self._menu_id = self._manager.new_merge_id ()
-        merge_path = '/tmw-menubar/view/subtitles/subtitle-download-placeholder'
-        self._manager.add_ui (self._menu_id,
-                             merge_path,
-                             'opensubtitles',
-                             'opensubtitles',
-                             Gtk.UIManagerItemType.MENUITEM,
-                             False
-                            )
-        self._action.set_visible (True)
-
-        self._manager.ensure_update ()
-
+        self._action = Gio.SimpleAction.new ("opensubtitles", None)
         self._action.connect ('activate', self._show_dialog)
+        self._totem.add_action (self._action)
 
-        self._action.set_sensitive (self._totem.is_playing () and
+        menu = self._totem.get_menu_section ("subtitle-download-placeholder")
+        menu.append (_(u'_Download Movie Subtitles…'), "app.opensubtitles")
+
+        self._action.set_enabled (self._totem.is_playing () and
                   self._check_allowed_scheme () and
                                   not self._check_is_audio ())
 
@@ -571,8 +543,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         return False
 
     def _delete_menu (self):
-        self._manager.remove_action_group (self._action_group)
-        self._manager.remove_ui (self._menu_id)
+        self._totem.empty_menu_section ("subtitle-download-placeholder")
 
     def _get_results (self, movie_hash, movie_size):
         self._list_store.clear ()
@@ -584,10 +555,10 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
 
         thread = SearchThread (self._model, movie_hash, movie_size)
         thread.start ()
-        GObject.idle_add (self._populate_treeview, thread)
+        GLib.idle_add (self._populate_treeview, thread)
 
         self._progress.set_text (_(u'Searching subtitles…'))
-        GObject.timeout_add (350, self._progress_bar_increment, thread)
+        GLib.timeout_add (350, self._progress_bar_increment, thread)
 
     def _populate_treeview (self, search_thread):
         if not search_thread.done:
@@ -618,19 +589,12 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
             subtitle_format = model.get_value (subtitle_iter, 1)
 
             if not filename:
-                bpath = xdg.BaseDirectory.xdg_cache_home + sep
+                bpath = GLib.get_user_cache_dir() + sep
                 bpath += 'totem' + sep
 
                 directory = Gio.file_new_for_path (bpath + 'subtitles' + sep)
-
                 if not directory.query_exists (None):
-                    if not path.exists (bpath):
-                        mkdir (bpath)
-                    if not path.exists (bpath + 'subtitles' + sep):
-                        mkdir (bpath + 'subtitles' + sep)
-                    # FIXME: We can't use this function until we depend on
-                    # GLib (PyGObject) 2.18
-                    # directory.make_directory_with_parents ()
+                    directory.make_directory_with_parents (None);
 
                 subtitle_file = Gio.file_new_for_path (self._filename)
                 movie_name = subtitle_file.get_basename ().rpartition ('.')[0]
@@ -696,7 +660,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
     def _download_and_apply (self):
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
-        self._action.set_sensitive (False)
+        self._action.set_enabled (False)
         self._tree_view.set_sensitive (False)
         self._save_selected_subtitle ()
 
@@ -708,7 +672,7 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
 
     # Callbacks
 
-    def __on_window__key_press_event (self, _widget, event):
+    def __on_window__key_press_event (self, _, event):
         if event.keyval == Gdk.KEY_Escape:
             self._close_dialog ()
             return True
@@ -720,13 +684,16 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         else:
             self._apply_button.set_sensitive (False)
 
-    def __on_treeview__row_activate (self, _tree_path, _column, _data):
+    def __on_treeview__row_activate (self,
+                                     _tree_path, # pylint: disable-msg=W0613
+                                     _column, # pylint: disable-msg=W0613
+                                     _data): # pylint: disable-msg=W0613
         self._download_and_apply ()
 
-    def __on_totem__file_opened (self, _totem, new_mrl):
+    def __on_totem__file_opened (self, _, new_mrl):
         # Check if allows subtitles
         if self._check_allowed_scheme () and not self._check_is_audio ():
-            self._action.set_sensitive (True)
+            self._action.set_enabled (True)
             if self._dialog:
                 self._find_button.set_sensitive (True)
                 # Check we're not re-opening the same file; if we are, don't
@@ -737,15 +704,15 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
                     self._list_store.clear ()
                     self._apply_button.set_sensitive (False)
         else:
-            self._action.set_sensitive (False)
+            self._action.set_enabled (False)
             if self._dialog and self._dialog.is_active ():
                 self._filename = None
                 self._list_store.clear ()
                 self._apply_button.set_sensitive (False)
                 self._find_button.set_sensitive (False)
 
-    def __on_totem__file_closed (self, _totem):
-        self._action.set_sensitive (False)
+    def __on_totem__file_closed (self, _):
+        self._action.set_enabled (False)
         if self._dialog:
             self._apply_button.set_sensitive (False)
             self._find_button.set_sensitive (False)
@@ -756,13 +723,13 @@ class OpenSubtitles (GObject.Object, # pylint: disable-msg=R0902
         self._model.lang = LANGUAGES[combo_model.get_value (combo_iter, 1)]
         self._settings.set_string ('language', self._model.lang)
 
-    def __on_close_clicked (self, _data):
+    def __on_close_clicked (self, _):
         self._close_dialog ()
 
-    def __on_apply_clicked (self, _data):
+    def __on_apply_clicked (self, _):
         self._download_and_apply ()
 
-    def __on_find_clicked (self, _data):
+    def __on_find_clicked (self, _):
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
         self._filename = self._totem.get_current_mrl ()
