@@ -51,7 +51,6 @@
 #include <libgd/gd.h>
 
 #define BROWSE_FLAGS          (GRL_RESOLVE_FAST_ONLY | GRL_RESOLVE_IDLE_RELAY)
-#define RESOLVE_FLAGS         (GRL_RESOLVE_FULL | GRL_RESOLVE_IDLE_RELAY)
 #define PAGE_SIZE             50
 #define SCROLL_GET_MORE_LIMIT 0.8
 
@@ -105,6 +104,7 @@ struct _TotemGriloPrivate {
 	GtkTreeModel *browser_filter_model;
 	gboolean in_search;
 	GList *metadata_keys;
+	guint thumbnail_update_id;
 
 	/* Search widgets */
 	GtkWidget *search_bar;
@@ -250,6 +250,18 @@ source_is_recent (GrlSource *source)
 	return strv_has_prefix (sources, id);
 }
 
+static gboolean
+source_is_adult (GrlSource *source)
+{
+	const char **tags;
+
+	tags = grl_source_get_tags (source);
+	if (!tags)
+		return FALSE;
+
+	return strv_has_prefix (tags, "adult");
+}
+
 static gchar *
 get_secondary_text (GrlMedia *media)
 {
@@ -451,6 +463,8 @@ update_search_thumbnails_idle (TotemGrilo *self)
 	GtkTreeModel *view_model, *model;
 	GtkIconView *icon_view;
 
+	self->priv->thumbnail_update_id = 0;
+
 	icon_view = GTK_ICON_VIEW (gd_main_view_get_generic_view (GD_MAIN_VIEW (self->priv->browser)));
 	if (!gtk_icon_view_get_visible_range (icon_view, &start_path, &end_path)) {
 		return FALSE;
@@ -516,7 +530,10 @@ update_search_thumbnails_idle (TotemGrilo *self)
 static void
 update_search_thumbnails (TotemGrilo *self)
 {
-	g_idle_add ((GSourceFunc) update_search_thumbnails_idle, self);
+	if (self->priv->thumbnail_update_id > 0)
+		return;
+	self->priv->thumbnail_update_id = g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) update_search_thumbnails_idle, self, NULL);
+	g_source_set_name_by_id (self->priv->thumbnail_update_id, "[totem] update_search_thumbnails_idle");
 }
 
 static void
@@ -574,7 +591,7 @@ add_local_metadata (TotemGrilo *self,
 	}
 
 	options = grl_operation_options_new (NULL);
-	grl_operation_options_set_flags (options, GRL_RESOLVE_FULL);
+	grl_operation_options_set_flags (options, GRL_RESOLVE_NORMAL);
 	grl_source_resolve_sync (self->priv->local_metadata_src,
 				 media,
 				 self->priv->metadata_keys,
@@ -1080,7 +1097,7 @@ find_media_cb (GtkTreeModel  *model,
 	if (g_strcmp0 (grl_media_get_id (media), grl_media_get_id (data->media)) == 0) {
 		g_object_unref (media);
 		data->found = TRUE;
-		data->iter = iter;
+		data->iter = gtk_tree_iter_copy (iter);
 		return TRUE;
 	}
 	g_object_unref (media);
@@ -1088,17 +1105,18 @@ find_media_cb (GtkTreeModel  *model,
 }
 
 static gboolean
-find_media (GtkTreeModel *model,
-	    GrlMedia     *media,
-	    GtkTreeIter  *iter)
+find_media (GtkTreeModel  *model,
+	    GrlMedia      *media,
+	    GtkTreeIter  **iter)
 {
 	FindMediaData data;
 
 	data.found = FALSE;
 	data.media = media;
+	data.iter = NULL;
 	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc) find_media_cb, &data);
 
-	*iter = *(data.iter);
+	*iter = data.iter;
 
 	return data.found;
 }
@@ -1125,10 +1143,12 @@ content_changed (TotemGrilo   *self,
 
 	for (i = 0; i < changed_medias->len; i++) {
 		GrlMedia *media = changed_medias->pdata[i];
-		GtkTreeIter iter;
+		GtkTreeIter *iter;
 
-		if (find_media (model, media, &iter))
-			update_media (GTK_TREE_STORE (model), &iter, source, media);
+		if (find_media (model, media, &iter)) {
+			update_media (GTK_TREE_STORE (model), iter, source, media);
+			gtk_tree_iter_free (iter);
+		}
 	}
 }
 
@@ -1144,10 +1164,20 @@ content_removed (TotemGrilo   *self,
 
 	for (i = 0; i < changed_medias->len; i++) {
 		GrlMedia *media = changed_medias->pdata[i];
-		GtkTreeIter iter;
+		GtkTreeIter *iter;
+		char *str;
 
-		if (find_media (model, media, &iter))
-			gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+		str = grl_media_serialize (media);
+		g_debug ("About to remove %s from the store", str);
+		g_free (str);
+
+		if (find_media (model, media, &iter)) {
+			gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
+			gtk_tree_iter_free (iter);
+		} else {
+			g_debug ("Could not find '%s' in the store",
+				 grl_media_get_id (media));
+		}
 	}
 }
 
@@ -1207,6 +1237,7 @@ source_added_cb (GrlRegistry *registry,
 	const char *id;
 
 	if (source_is_blacklisted (source) ||
+	    source_is_adult (source) ||
 	    !(grl_source_get_supported_media (source) & GRL_MEDIA_TYPE_VIDEO)) {
 		grl_registry_unregister_source (registry,
 		                                source,
@@ -1263,7 +1294,8 @@ source_added_cb (GrlRegistry *registry,
 		}
 
 		if (monitor &&
-		    (ops & GRL_OP_NOTIFY_CHANGE)) {
+		    (ops & GRL_OP_NOTIFY_CHANGE) &&
+		    g_str_equal (id, "grl-filesystem") == FALSE) {
 			grl_source_notify_change_start (source, NULL);
 			g_signal_connect (G_OBJECT (source), "content-changed",
 					  G_CALLBACK (content_changed_cb), self);
@@ -2024,7 +2056,12 @@ delete_foreach (gpointer data,
 	gboolean success;
 	GError *error = NULL;
 
-	gtk_tree_model_get_iter (view_model, &iter, path);
+	GtkTreeModel *model;
+	GtkTreeIter real_model_iter;
+
+	if (!gtk_tree_model_get_iter (view_model, &iter, path))
+		return;
+
 	gtk_tree_model_get (view_model, &iter,
 			    MODEL_RESULTS_CONTENT, &media,
 			    MODEL_RESULTS_SOURCE, &source,
@@ -2035,12 +2072,13 @@ delete_foreach (gpointer data,
 		g_debug ("Removing item '%s' through Grilo",
 			 grl_media_get_id (media));
 		grl_source_remove_sync (source, media, &error);
-		success = (error != NULL);
+		success = (error == NULL);
 	} else {
 		const char *uri;
 
 		uri = grl_media_get_url (media);
 		if (!uri) {
+			success = FALSE;
 			g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
 					     "Item cannot be removed through Grilo and doesn't have a URI, please file a bug");
 		} else {
@@ -2057,25 +2095,27 @@ delete_foreach (gpointer data,
 			   grl_media_get_id (media),
 			   error->message);
 		g_error_free (error);
+		goto end;
+	}
+	if (grl_source_supported_operations (source) & GRL_OP_REMOVE)
+		goto end;
+
+	/* In case of success, or if we didn't remove the item through the source */
+	if (GTK_IS_TREE_MODEL_FILTER (view_model)) {
+		model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (view_model));
+		gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (view_model),
+								  &real_model_iter, &iter);
+	} else if (GTK_IS_TREE_MODEL_SORT (view_model)) {
+		model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (view_model));
+		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (view_model),
+								&real_model_iter, &iter);
 	} else {
-		GtkTreeModel *model;
-		GtkTreeIter real_model_iter;
-
-		if (GTK_IS_TREE_MODEL_FILTER (view_model)) {
-			model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (view_model));
-			gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (view_model),
-									  &real_model_iter, &iter);
-		} else if (GTK_IS_TREE_MODEL_SORT (view_model)) {
-			model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (view_model));
-			gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (view_model),
-									&real_model_iter, &iter);
-		} else {
-			g_assert_not_reached ();
-		}
-
-		gtk_tree_store_remove (GTK_TREE_STORE (model), &real_model_iter);
+		g_assert_not_reached ();
 	}
 
+	gtk_tree_store_remove (GTK_TREE_STORE (model), &real_model_iter);
+
+end:
 	g_clear_object (&media);
 	g_clear_object (&source);
 }
@@ -2087,11 +2127,15 @@ delete_cb (TotemSelectionToolbar *bar,
 	GtkTreeModel *model;
 	GList *list;
 
+	g_signal_handlers_block_by_func (self->priv->browser, "view-selection-changed", self);
+
 	model = gd_main_view_get_model (GD_MAIN_VIEW (self->priv->browser));
 	list = gd_main_view_get_selection (GD_MAIN_VIEW (self->priv->browser));
 	g_list_foreach (list, delete_foreach, model);
 	g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
 
+	g_signal_handlers_unblock_by_func (self->priv->browser, "view-selection-changed", self);
+
 	g_object_set (G_OBJECT (self->priv->browser), "selection-mode", FALSE, NULL);
 }
 
@@ -2248,6 +2292,35 @@ remaining_to_text (GtkTreeViewColumn *column,
 	g_free (text);
 }
 
+static void
+media_to_text (GtkTreeViewColumn *column,
+	       GtkCellRenderer   *cell,
+	       GtkTreeModel      *model,
+	       GtkTreeIter       *iter,
+	       gpointer           user_data)
+{
+	int column_num;
+	GrlMedia *media;
+	const char *text;
+
+	column_num = GPOINTER_TO_INT (user_data);
+
+	gtk_tree_model_get (model, iter, MODEL_RESULTS_CONTENT, &media, -1);
+	if (media == NULL)
+		return;
+
+	if (column_num == GD_MAIN_COLUMN_ID)
+		text = grl_media_get_id (media);
+	else if (column_num == GD_MAIN_COLUMN_URI)
+		text = grl_media_get_url (media);
+	else
+		g_assert_not_reached ();
+
+	g_object_set (cell, "text", text, NULL);
+	g_object_unref (media);
+
+}
+
 static void
 create_debug_window (TotemGrilo       *self,
 		     GtkTreeModel     *model)
@@ -2271,12 +2344,12 @@ create_debug_window (TotemGrilo       *self,
 	gtk_widget_grab_focus (GTK_WIDGET (tree));
 	gtk_container_add (GTK_CONTAINER (scrolled), tree);
 
-	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW (tree), -1,
+	gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree), -1,
 						    "ID", gtk_cell_renderer_text_new (),
-						    "text", GD_MAIN_COLUMN_ID, NULL);
-	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW (tree), -1,
+						    media_to_text, GINT_TO_POINTER (GD_MAIN_COLUMN_ID), NULL);
+	gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree), -1,
 						    "URI", gtk_cell_renderer_text_new (),
-						    "text", GD_MAIN_COLUMN_URI, NULL);
+						    media_to_text, GINT_TO_POINTER (GD_MAIN_COLUMN_URI), NULL);
 	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW (tree), -1,
 						    "Primary text", gtk_cell_renderer_text_new (),
 						    "text", GD_MAIN_COLUMN_PRIMARY_TEXT, NULL);
@@ -2344,6 +2417,11 @@ totem_grilo_finalize (GObject *object)
 	GList *s;
 	GrlRegistry *registry;
 
+	if (self->priv->thumbnail_update_id > 0) {
+		g_source_remove (self->priv->thumbnail_update_id);
+		self->priv->thumbnail_update_id = 0;
+	}
+
 	registry = grl_registry_get_default ();
 	g_signal_handlers_disconnect_by_func (registry, source_added_cb, self);
 	g_signal_handlers_disconnect_by_func (registry, source_removed_cb, self);
